/*
 * Copyright (C) 2003-2009 eXo Platform SAS.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.reflext.api;

import java.lang.annotation.Annotation;

/**
 * An annotated target.
 *
 * @author <a href="mailto:julien.viet@exoplatform.com">Julien Viet</a>
 * @version $Revision$
 * @param <T> the target type
 * @param <A> the annotation type
 */
public class Annotated<T, A extends Annotation> {

  /** . */
  private final T target;

  /** . */
  private final A annotation;

  /**
   * Create a new annotated target.
   *
   * @param target the target
   * @param annotation the annotation
   * @throws NullPointerException if any argument is null
   */
  public Annotated(T target, A annotation) throws NullPointerException {
    if (target == null) {
      throw new NullPointerException("No null target accepted");
    }
    if (annotation == null) {
      throw new NullPointerException("No null annotation accepted");
    }

    //
    this.target = target;
    this.annotation = annotation;
  }

  public T getTarget() {
    return target;
  }

  public A getAnnotation() {
    return annotation;
  }
}
